import datetime
import os
import shutil
import tempfile
import unittest
from zope.component import queryUtility, getUtilitiesFor
from zope.interface.verify import verifyObject, verifyClass
from waeup.kofa.interfaces import ICSVExporter
from waeup.kofa.testing import KofaUnitTestLayer
from waeup.kofa.applicants import ApplicantsContainer
from waeup.kofa.applicants.export import ApplicantsContainerExporter
from waeup.kofa.applicants.interfaces import (
    IApplicantsContainerProvider, AppCatSource)

class ApplicantsContainersExporterTest(unittest.TestCase):

    layer = KofaUnitTestLayer

    def setUp(self):
        self.workdir = tempfile.mkdtemp()
        self.outfile = os.path.join(self.workdir, 'myoutput.csv')
        return

    def tearDown(self):
        shutil.rmtree(self.workdir)
        return

    def test_ifaces(self):
        # make sure we fullfill interface contracts
        obj = ApplicantsContainerExporter()
        verifyObject(ICSVExporter, obj)
        verifyClass(ICSVExporter, ApplicantsContainerExporter)
        return

    def test_get_as_utility(self):
        # we can get a faculty exporter as utility
        result = queryUtility(ICSVExporter, name="applicantscontainers")
        self.assertTrue(result is not None)
        return

    def setup_container(self, container):
        # set all attributes of a container
        container.code = u'dp2012'
        container.title = u'General Studies 2012/13'
        from waeup.kofa.applicants.interfaces import (
            ApplicationTypeSource, ApplicantContainerProviderSource,)
        container.prefix = list(ApplicationTypeSource()(container))[0]
        container.year = 2012
        provider = [
            x for x in getUtilitiesFor(IApplicantsContainerProvider)
            if x[0] == 'waeup.kofa.applicants.ApplicantsContainer'][0]
        container.provider = provider
        container.application_category = list(AppCatSource()(container))[0]
        container.description = u'Some Description\nwith linebreak\n'
        container.description += u'<<de>>man spriht deutsh'
        container.startdate = datetime.date(2012, 1, 1)
        container.enddate = datetime.date(2012, 1, 31)
        return container

    def test_export(self):
        # we can export a set of applicants containers (w/o applicants)
        container = ApplicantsContainer()
        container = self.setup_container(container)
        exporter = ApplicantsContainerExporter()
        exporter.export([container], self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'code,title,prefix,entry_level,year,provider,application_category,'
            'description,startdate,enddate,strict_deadline\r\n'

            'dp2012,General Studies 2012/13,app,100,2012,'
            'waeup.kofa.applicants.ApplicantsContainer,basic,'
            '"Some Description\nwith linebreak\n<<de>>man spriht deutsh",'
            '2012-01-01,2012-01-31,1\r\n'
            )
        return
